/*
 * BCM7038 serial console and serial debug
 *
 * Copyright (C) 2004 TiVo Inc. All Rights Reserved.
 */

#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/irq.h>
#include <linux/sched.h>
#include <linux/console.h>
#include <linux/config.h>
#include <asm/gdb-stub.h>
#include <asm/bcm7038.h>

#define UART_REG(c,offset) ((volatile unsigned long *) \
   (BCM_UARTA_BASE + ((c)->index * 0x20) + (offset*4)))

#ifdef CONFIG_TIVO_DEVEL
//
// Serial kernel debug support
//

extern void do_async_dbg(struct pt_regs *);

static int __initdata nodebug;
static int debug_index = 1;
static int __initdata console_index = 0;

#define DEBUG_UART_REG(offset) ((volatile unsigned long *) \
    (BCM_UARTA_BASE + (debug_index * 0x20) + (offset*4)))

#define CONSOLE_UART_REG(offset) ((volatile unsigned long *) \
    (BCM_UARTA_BASE + (console_index * 0x20) + (offset*4)))

static int __init nodebug_setup(char *str)
{
  nodebug = 1;
  return 1;
}

__setup("nodebug", nodebug_setup);

int putDebugChar(unsigned char c)
{
  while ((*DEBUG_UART_REG(BCM_UART_XMTSTAT) & 0x01) == 0);


  *DEBUG_UART_REG(BCM_UART_XMTDATA) = c;
  return 1;
}

unsigned char getDebugChar(void)
{
  while ((*DEBUG_UART_REG(BCM_UART_RCVSTAT) & 0x04) == 0);

  return (unsigned char)(*DEBUG_UART_REG(BCM_UART_RCVDATA));
}

int pollDebugChar(void)
{
	return ((*DEBUG_UART_REG(BCM_UART_RCVSTAT) & 0x04) != 0);
}

static void dbgs_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	unsigned long flags;

	save_and_cli(flags);
	do_async_dbg(regs);
	restore_flags(flags);
}

static int __init init_dbgs(void)
{
	*DEBUG_UART_REG(BCM_UART_CONTROL) = 0x16;
	*DEBUG_UART_REG(BCM_UART_RCVSTAT) = 0;
	*DEBUG_UART_REG(BCM_UART_BAUDHI) = 0;
	*DEBUG_UART_REG(BCM_UART_BAUDLO) = 0x0e;

	if (nodebug == 0) {
		if (request_irq(debug_index == 0 ? BCM_IRQ_UARTA_CPU_INTR
                                                 : BCM_IRQ_UARTB_CPU_INTR,
                                dbgs_interrupt, SA_SHIRQ | SA_INTERRUPT,
		                "debug", "debug") == 0) {
		    *DEBUG_UART_REG(BCM_UART_RCVSTAT) = 0x02;
		}
	}
#ifdef CONFIG_REMOTE_DEBUG
	set_debug_traps();
#endif
	return 0;
}

__initcall(init_dbgs);
#endif // CONFIG_TIVO_DEVEL

//
// Serial console support
//

static int __initdata dsscon = 1;

static int __init dsscon_setup(char *str)
{
	if (strcmp(str, "true") == 0)
		dsscon = 1;
	else
		dsscon = 0;
	return 0;
}

__setup("dsscon=", dsscon_setup);

static void early_debug_write(struct console *co, const char *buf, unsigned len)
{
	int i;
	for (i = 0; i < len; i++) {
		while ((*UART_REG(co, BCM_UART_XMTSTAT) & 0x01) == 0) {
		}
		*UART_REG(co, BCM_UART_XMTDATA) = buf[i];
		if (buf[i] == 10) {
			while ((*UART_REG(co, BCM_UART_XMTSTAT) & 0x01) == 0) {
			}
			*UART_REG(co, BCM_UART_XMTDATA) = 13;
		}
	}
}

static int early_debug_setup(struct console *co, char *unused)
{
//	if (co->index > 1) {
//		return -1;
//	}
#ifdef CONFIG_TIVO_DEVEL
	debug_index = co->index;
#endif

	*UART_REG(co, BCM_UART_CONTROL) = 0x16;
	*UART_REG(co, BCM_UART_BAUDHI) = 0;
	*UART_REG(co, BCM_UART_BAUDLO) = 0x0e;

	return 0;
}

static struct console early_debug_console = {
	.name = { "ttyS" },
	.write = early_debug_write,
	.setup = early_debug_setup,
	.flags = CON_PRINTBUFFER,
	.index = -1,
};

void __init gen05_console_init(void)
{
	if (dsscon) {
	    register_console(&early_debug_console);
	}
}

static int gen05_sercons_ready = 0;

int __init sercons_poll( void )
{
       if ( gen05_sercons_ready < 0 )
          return -1;

       if ( !gen05_sercons_ready ) {
           gen05_console_init();
	   gen05_sercons_ready = 1;
       } 

       if ( *CONSOLE_UART_REG(BCM_UART_RCVSTAT) & 0x4 )
          return *CONSOLE_UART_REG(BCM_UART_RCVDATA) & 0xFF;

       return -1;

}

void __init uart_init( unsigned long uClkSpeed )
{
  unsigned long uBaudRate;

  uBaudRate = uClkSpeed / 115200 / 16;
  *CONSOLE_UART_REG(BCM_UART_BAUDLO)  = (uBaudRate & 0xFF);
  *CONSOLE_UART_REG(BCM_UART_BAUDHI)  = (uBaudRate >> 8) & 0xFF;
  *DEBUG_UART_REG(BCM_UART_BAUDLO)    = (uBaudRate & 0xFF);
  *DEBUG_UART_REG(BCM_UART_BAUDHI)    = (uBaudRate >> 8) & 0xFF;
  *CONSOLE_UART_REG(BCM_UART_CONTROL) = RXEN | TXEN | BITM8;
  *DEBUG_UART_REG(BCM_UART_CONTROL)   = RXEN | TXEN | BITM8;
  
}

#ifdef CONFIG_TIVO_DEVEL

void __init uart_putc( char cData )
{
  while( !(*CONSOLE_UART_REG(BCM_UART_XMTSTAT) & TXDRE) )
    barrier();

  *CONSOLE_UART_REG(BCM_UART_XMTDATA) = cData;
}

void __init uart_puts( const char *pString )
{
  while( *pString ) {
    if( *pString == '\n' )
      uart_putc( '\r' );
    uart_putc( *pString++ );
  }
}
#endif
