#ifndef BCMENET_REGS_H
#define BCMENET_REGS_H   

/**
 * IUDMA channel control register 
 */
#define IUDMA_CHNL_CTL_KICKED_SHIFT              0x0
#define IUDMA_CHNL_CTL_KICKED_MASK               0x00000001UL
#define IUDMA_CHNL_CTL_KICKED                    0x00000001UL

#define IUDMA_CHNL_CTL_PKT_HALT_SHIFT            0x5
#define IUDMA_CHNL_CTL_PKT_HALT_MASK             0x00000020UL
#define IUDMA_CHNL_CTL_PKT_HALT                  0x00000020UL

#define IUDMA_CHNL_CTL_BURST_HALT_SHIFT          0x6
#define IUDMA_CHNL_CTL_BURST_HALT_MASK           0x00000040UL
#define IUDMA_CHNL_CTL_BURST_HALT                0x00000040UL

/**
 * IUDMA channel IRQ status register
 */
#define IUDMA_CHNL_IRQ_STATUS_BDONE_SHIFT       0x0
#define IUDMA_CHNL_IRQ_STATUS_BDONE_MASK        0x00000001UL
#define IUDMA_CHNL_IRQ_STATUS_BDONE             0x00000001UL

#define IUDMA_CHNL_IRQ_STATUS_PDONE_SHIFT       0x1
#define IUDMA_CHNL_IRQ_STATUS_PDONE_MASK        0x00000002UL
#define IUDMA_CHNL_IRQ_STATUS_PDONE             0x00000002UL

#define IUDMA_CHNL_IRQ_STATUS_NOTVID_SHIFT      0x2
#define IUDMA_CHNL_IRQ_STATUS_NOTVID_MASK       0x00000004UL
#define IUDMA_CHNL_IRQ_STATUS_NOTVID            0x00000004UL


/**
 * IUDMA channel IRQ mask register
 */
#define IUDMA_CHNL_IRQ_MASK_BDONE_SHIFT         0x0
#define IUDMA_CHNL_IRQ_MASK_BDONE_MASK          0x00000001UL
#define IUDMA_CHNL_IRQ_MASK_BDONE               0x00000001UL

#define IUDMA_CHNL_IRQ_MASK_PDONE_SHIFT         0x1
#define IUDMA_CHNL_IRQ_MASK_PDONE_MASK          0x00000002UL
#define IUDMA_CHNL_IRQ_MASK_PDONE               0x00000002UL

#define IUDMA_CHNL_IRQ_MASK_NOTVID_SHIFT        0x2
#define IUDMA_CHNL_IRQ_MASK_NOTVID_MASK         0x00000004UL
#define IUDMA_CHNL_IRQ_MASK_NOTVID              0x00000004UL


/**
 * IUDMA controller config register
 */
#define IUDMA_CNTRL_CFG_IUDMA_EN_SHIFT          0
#define IUDMA_CNTRL_CFG_IUDMA_EN_MASK           0x00000001UL
#define IUDMA_CNTRL_CFG_IUDMA_EN                0x00000001UL

#define IUDMA_CNTRL_CFG_FC_EN_SHIFT             1
#define IUDMA_CNTRL_CFG_FC_EN_MASK              0x00000002UL
#define IUDMA_CNTRL_CFG_FC_EN                   0x00000002UL
/**
 * IUDMA Channel flow threshold lo register
 */
#define IUDMA_CHNL_FLOW_THRESH_LO_SHIFT         0x0
#define IUDMA_CHNL_FLOW_THRESH_LO_MASK          0x000000ffUL

/**
 * IUDMA Channel flow threshold hi register
 */
#define IUDMA_CHNL_FLOW_THRESH_HI_SHIFT         0x0
#define IUDMA_CHNL_FLOW_THRESH_HI_MASK          0x000000ffUL


/**
 * IUDMA Channel flow alloc register
 */
#define IUDMA_CHNL_FLOW_ALLOC_BUFFALLOC_SHIFT   0x0
#define IUDMA_CHNL_FLOW_ALLOC_BUFFALLOC_MASK    0x000000ffUL

#define IUDMA_CHNL_FLOW_ALLOC_FORCE_SHIFT       31
#define IUDMA_CHNL_FLOW_ALLOC_FORCE_MASK        0x80000000UL
#define IUDMA_CHNL_FLOW_ALLOC_FORCE             0x80000000UL

/**
 * IUDMA R5K IRQ status register
 */
#define IUDMA_R5K_IRQ_STATUS_SCB_SHIFT          0
#define IUDMA_R5K_IRQ_STATUS_SCB_MASK           0x00000001UL
#define IUDMA_R5K_IRQ_STATUS_SCB                0x00000001UL

#define IUDMA_R5K_IRQ_STATUS_IUDMA_SHIFT        1
#define IUDMA_R5K_IRQ_STATUS_IUDMA_MASK         0x00000002UL
#define IUDMA_R5K_IRQ_STATUS_IUDMA              0x00000002UL

#define IUDMA_R5K_IRQ_STATUS_EMAC_SHIFT         2
#define IUDMA_R5K_IRQ_STATUS_EMAC_MASK          0x00000002UL
#define IUDMA_R5K_IRQ_STATUS_EMAC               0x00000002UL

#define IUDMA_R5K_IRQ_STATUS_EPHY_SHIFT         3
#define IUDMA_R5K_IRQ_STATUS_EPHY_MASK          0x00000004UL
#define IUDMA_R5K_IRQ_STATUS_EPHY               0x00000004UL

#define IUDMA_R5K_IRQ_STATUS_GISB_SHIFT         4
#define IUDMA_R5K_IRQ_STATUS_GISB_MASK          0x00000010UL
#define IUDMA_R5K_IRQ_STATUS_GISB               0x00000010UL

/**
 * IUDMA R5K IRQ MASK register
 */
#define IUDMA_R5K_IRQ_MASK_SCB_SHIFT            0
#define IUDMA_R5K_IRQ_MASK_SCB_MASK             0x00000001UL
#define IUDMA_R5K_IRQ_MASK_SCB                  0x00000001UL

#define IUDMA_R5K_IRQ_MASK_IUDMA_SHIFT          1
#define IUDMA_R5K_IRQ_MASK_IUDMA_MASK           0x00000002UL
#define IUDMA_R5K_IRQ_MASK_IUDMA                0x00000002UL

#define IUDMA_R5K_IRQ_MASK_EMAC_SHIFT           2
#define IUDMA_R5K_IRQ_MASK_EMAC_MASK            0x00000002UL
#define IUDMA_R5K_IRQ_MASK_EMAC                 0x00000002UL

#define IUDMA_R5K_IRQ_MASK_EPHY_SHIFT           3
#define IUDMA_R5K_IRQ_MASK_EPHY_MASK            0x00000004UL
#define IUDMA_R5K_IRQ_MASK_EPHY                 0x00000004UL

#define IUDMA_R5K_IRQ_MASK_GISB_SHIFT           4
#define IUDMA_R5K_IRQ_MASK_GISB_MASK            0x00000010UL
#define IUDMA_R5K_IRQ_MASK_GISB                 0x00000010UL


/**
 * RX Config register
 */
#define RX_CONFIG_DISB_SHIFT                    0
#define RX_CONFIG_DISB_MASK                     0x00000001UL
#define RX_CONFIG_DISB                          0x00000001UL

#define RX_CONFIG_ACCMULT_SHIFT                 1
#define RX_CONFIG_ACCMULT_MASK                  0x00000002UL
#define RX_CONFIG_ACCMULT                       0x00000002UL

#define RX_CONFIG_RDT_SHIFT                     2
#define RX_CONFIG_RDT_MASK                      0x00000004UL
#define RX_CONFIG_RDT                           0x00000004UL

#define RX_CONFIG_PROM_SHIFT                    3
#define RX_CONFIG_PROM_MASK                     0x00000008UL
#define RX_CONFIG_PROM                          0x00000008UL

#define RX_CONFIG_LPBK_SHIFT                    4
#define RX_CONFIG_LPBK_MASK                     0x00000010UL
#define RX_CONFIG_LPBK                          0x00000010UL

#define RX_CONFIG_ENFLOW_SHIFT                  5
#define RX_CONFIG_ENFLOW_MASK                   0x00000020UL
#define RX_CONFIG_ENFLOW                        0x00000020UL

#define RX_CONFIG_UNIFLOW_SHIFT                 6
#define RX_CONFIG_UNIFLOW_MASK                  0x00000040UL
#define RX_CONFIG_UNIFLOW                       0x00000040UL

#define RX_CONFIG_PMREJ_SHIFT                   7
#define RX_CONFIG_PMREJ_MASK                    0x00000080UL
#define RX_CONFIG_PMREJ                         0x00000080UL

#define RX_CONFIG_AGRDMA_SHIFT                  8
#define RX_CONFIG_AGRDMA_MASK                   0x00000100UL
#define RX_CONFIG_AGRDMA                        0x00000100UL

/**
 * RX LENGTH register
 */
#define RX_LENGTH_RML_SHIFT                     0
#define RX_LENGTH_RML_MASK                      0x000007FFUL

/**
 * TX LENGTH register
 */
#define TX_LENGTH_RML_SHIFT                     0
#define TX_LENGTH_RML_MASK                      0x000007FFUL

/**
 * ENET TOP EMAC Interrupt mask register
 */
#define EMAC_MASK_MII_SHIFT                     0
#define EMAC_MASK_MII_MASK                      0x00000001UL
#define EMAC_MASK_MII                           0x00000001UL

#define EMAC_MASK_MIB_SHIFT                     1
#define EMAC_MASK_MIB_MASK                      0x00000002UL
#define EMAC_MASK_MIB                           0x00000002UL

#define EMAC_MASK_FLOW_SHIFT                    2
#define EMAC_MASK_FLOW_MASK                     0x00000004UL
#define EMAC_MASK_FLOW                          0x00000004UL


/**
 * ENET TOP EMAC Interrupt status register
 */
#define EMAC_STATUS_MII_SHIFT                   0
#define EMAC_STATUS_MII_MASK                    0x00000001UL
#define EMAC_STATUS_MII                         0x00000001UL

#define EMAC_STATUS_MIB_SHIFT                   1
#define EMAC_STATUS_MIB_MASK                    0x00000002UL
#define EMAC_STATUS_MIB                         0x00000002UL

#define EMAC_STATUS_FLOW_SHIFT                  2
#define EMAC_STATUS_FLOW_MASK                   0x00000004UL
#define EMAC_STATUS_FLOW                        0x00000004UL

/**
 * ENET top control register
 */
#define ENET_CONTROL_EE_SHIFT                   0
#define ENET_CONTROL_EE_MASK                    0x00000001UL
#define ENET_CONTROL_EE                         0x00000001UL

#define ENET_CONTROL_ED_SHIFT                   1
#define ENET_CONTROL_ED_MASK                    0x00000002UL
#define ENET_CONTROL_ED                         0x00000002UL

#define ENET_CONTROL_EP_SHIFT                   3 /* bit 2 is resv */
#define ENET_CONTROL_EP_MASK                    0x00000008UL
#define ENET_CONTROL_EP                         0x00000008UL


/**
 * ENET top xmit control register
 */
#define ENET_XMIT_CONTROL_FD_SHIFT              0
#define ENET_XMIT_CONTROL_FD_MASK               0x00000001UL
#define ENET_XMIT_CONTROL_FD                    0x00000001UL

#define ENET_XMIT_CONTROL_FM_SHIFT              1
#define ENET_XMIT_CONTROL_FM_MASK               0x00000002UL
#define ENET_XMIT_CONTROL_FM                    0x00000002UL

#define ENET_XMIT_CONTROL_SB_SHIFT              2
#define ENET_XMIT_CONTROL_SB_MASK               0x00000004UL
#define ENET_XMIT_CONTROL_SB                    0x00000004UL

#define ENET_XMIT_CONTROL_SS_SHIFT              3
#define ENET_XMIT_CONTROL_SS_MASK               0x00000008UL
#define ENET_XMIT_CONTROL_SS                    0x00000008UL


/**
 * ENET top watermark register
 */
#define ENET_WATERMARK_WM_SHIFT                 0
#define ENET_WATERMARK_WM_MASK                  0x000000ffUL

/**
 * ENET mii_sc register
 */
#define ENET_MII_SC_MDC_SHIFT                   0
#define ENET_MII_SC_MDC_MASK                    0x0000003fUL

#define ENET_MII_SC_PRE_EN_SHIFT                7
#define ENET_MII_SC_PRE_EN_MASK                 0x00000080UL
#define ENET_MII_SC_PRE_EN                      0x00000080UL

#define ENET_MII_SC_IDLE_EN_SHIFT               31 
#define ENET_MII_SC_IDLE_EN_MASK                0x80000000UL
#define ENET_MII_SC_IDLE_EN                     0x80000000UL



#endif /* BCMENET_REGS_H */
